﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DAL;

namespace MenuLibrary
{
    public class Menu
    {
        public string Name { get; set; }
        public List<MenuItem> Items { get; set; }
        public bool StartWithClearScreen { get; set; }
        public Action<Puzzle15Engine.Puzzle15Engine,AppDbContext> ActionToDoFirst { get; set; }
        public void DoMenu(Puzzle15Engine.Puzzle15Engine puzzle15Engine, AppDbContext db)
        {
            for (;;)
            {
                if (StartWithClearScreen)
                {
                    Console.Clear();
                }

                ActionToDoFirst?.Invoke(puzzle15Engine, db);

                OutputMenu();
                var c = GetCommand();

                if (c == "X")
                {
                    //exit from loop
                    break;
                }

                var item = Items.FirstOrDefault(a => a.Command == c);
                if (item == null)
                {
                    Console.WriteLine("No such command!");
                }
                else
                {
                    if (item.MethodToExecute == null)
                    {
                        Console.WriteLine("No method assigned to delegate command!");
                    }
                    else
                    {
                        item.MethodToExecute(puzzle15Engine, db);
                    }
                }


            }
        }

        private void OutputMenu()
        {
            Console.WriteLine($"---- {Name} ----");
            foreach (var menuItem in Items)
            {
                Console.Write(menuItem.Command);
                Console.Write(" - ");
                Console.WriteLine(menuItem.Value);
            }
            Console.WriteLine("X - return");
        }

        private string GetCommand(bool trim = true, bool toLower = false, bool toUpper = true)
        {
            if (toLower && toUpper)
            {
                throw new ArgumentException("ToLower and ToUpper cannot be true at the same time!");
            }

            Console.Write(">");
            var res = Console.ReadLine();
            if (res != null && trim)
            {
                res = res.Trim();
            }
            if (toLower)
            {
                res = res?.ToLower();
            }
            if (toUpper)
            {
                res = res?.ToUpper();
            }

            return res;

        }
    }
}
